//
//  RADisplaySwapper.m
//  DisplaySwapper
//
//  Created by Quentin Carnicelli on Sat Mar 27 2004.
//  Copyright (c) 2004 Quentin D. Carnicelli. All rights reserved.
//

#import "RADisplaySwapper.h"

#import "RAWindowAnimation.h"

@implementation RADisplaySwapper

static id _sharedSwapper = nil;

+ (id)sharedSwapper
{
	if( _sharedSwapper == nil )
		_sharedSwapper = [[self alloc] init];
	
	return _sharedSwapper;
}

- (NSArray*)windowList
{
	int numWindows;
	int* windowList;
	int i;
	NSWindow* window;
	NSMutableArray* allWindows = [NSMutableArray array];
	
	NSCountWindows( &numWindows );
	
	windowList = malloc( numWindows * sizeof(int) );
	NSWindowList( numWindows, windowList );

	for( i = 0; i < numWindows; i++ )
	{
		window = [[NSApplication sharedApplication] windowWithWindowNumber: windowList[i]];
		if( window )
			[allWindows addObject: window];
	}

	free( windowList );
	
	return allWindows;
}

- (NSPoint)originForWindow: (NSWindow*)window onScreen: (NSScreen*)newScreen
{
	NSRect newScreenFrame;
	NSScreen* oldScreen;
	NSRect oldScreenFrame;
	NSPoint oldOrigin;
	NSPoint newOrigin;
	float scaleX, scaleY;
		
	oldScreen = [window screen];
	oldScreenFrame = [oldScreen frame];
	oldOrigin = [window frame].origin;

	if( newScreen == oldScreen )
		return oldOrigin;
		
	newScreenFrame = [newScreen frame];

	//scaleX = (newScreenFrame.size.width/oldScreenFrame.size.width);
	//scaleY = (newScreenFrame.size.height/oldScreenFrame.size.height);
	scaleX = scaleY = 1;
	
	newOrigin.x = (oldOrigin.x-oldScreenFrame.origin.x) * scaleX + newScreenFrame.origin.x;
	newOrigin.y = (oldOrigin.y-oldScreenFrame.origin.y) * scaleY + newScreenFrame.origin.y;
		
	return newOrigin;
}

- (void)swapScreen: (NSScreen*)scrA withScreen: (NSScreen*)scrB animationTime: (float)duration
{
	NSEnumerator* windowsEnum = [[self windowList] objectEnumerator];
	NSWindow* window;
	NSScreen* windowScreen;
	NSScreen* dstScreen;
	RAWindowAnimationList* animationList;
	
	animationList = [[RAWindowAnimationList alloc] init];
	
	while( (window = [windowsEnum nextObject]) != nil )
	{
		windowScreen = [window screen];
		
		if( windowScreen == scrA )
			dstScreen = scrB;
		else if( windowScreen == scrB )
			dstScreen = scrA;
		else
			dstScreen = nil;
	
		if( dstScreen )
		{
			NSPoint newOrigin;
			RAWindowAnimationMoveAction* action;
			
			newOrigin = [self originForWindow: window onScreen: dstScreen];
			action = [[[RAWindowAnimationMoveAction alloc] initWithWindow: window newOrigin: newOrigin] autorelease];
			[animationList addAction: action];
		}
	}

	[animationList animate: duration];
	[animationList release];

}


@end
