//
//  RADisplaySwapper.m
//  DisplaySwapper
//
//  Created by Quentin Carnicelli on Sat Mar 27 2004.
//  Copyright (c) 2004 Quentin D. Carnicelli. All rights reserved.
//

#import "RADisplaySwapperPlugin.h"
#import "RADisplaySwapper.h"

@implementation RADisplaySwapperPlugin

static id _sharedInstance = nil;

+ (void)pluginDidLoad: (NSBundle*)bundle
{
	if( _sharedInstance == nil )
	{
		if( [[NSScreen screens] count] <= 1 ) //Don't even both
			return;

		if( [[[NSApplication sharedApplication] currentEvent] modifierFlags] & NSShiftKeyMask ) //Bail if Shift is down
			return;

		_sharedInstance = [[self alloc] init];
	}
}

- (id)init
{
	self = [super init];
	
	if( self )
	{
		[self _insertMenus];
	}
	
	return self;
}

- (void)_insertMenus
{
	NSArray* allScreens = [NSScreen screens];
	int numberOfScreens = [allScreens count];
	NSMenu* menu;
	NSString* title;
	NSMenuItem* item;
	int i;
	int screenA, screenB;
	NSArray* screenPair;
	
	if( numberOfScreens <= 1 )
		return;
	
	menu = [[NSApplication sharedApplication] mainMenu];
	menu = [[menu itemWithTitle: @"Window"] submenu];
		
	[menu insertItem: [NSMenuItem separatorItem] atIndex: 2];
	
	i = 0;
	for( screenA = 0; screenA < numberOfScreens; screenA++ )
	{
		for( screenB = screenA+1; screenB < numberOfScreens; screenB++ )
		{
			title = [NSString stringWithFormat: @"Swap Display %d and %d", screenA+1, screenB+1];
			item = (NSMenuItem*)[menu insertItemWithTitle: title action: nil keyEquivalent: @"" atIndex: i+3];
			
			[item setTarget: self];
			[item setAction: @selector( _hitSwapMenuItem: )];
			[item setTag: 'SWP0'+i]; //We need to set a tag for Xcodes Key Binding system
			
			screenPair = [NSArray arrayWithObjects:
							[allScreens objectAtIndex: screenA],
							[allScreens objectAtIndex: screenB],
							nil];
			[item setRepresentedObject: screenPair ];
			
			i++;
		}
	}
}

- (BOOL)validateUserInterfaceItem: (id)item
{
	SEL action = [item action];
	
	if( action == @selector( _hitSwapMenuItem: ) )
		return [[[RADisplaySwapper sharedSwapper] windowList] count] >= 1;
		
	return YES;
}

- (void)_hitSwapMenuItem: (id)sender
{
	NSArray*  screenPair = [sender representedObject];
	NSScreen* screenA = [screenPair objectAtIndex: 0];
	NSScreen* screenB = [screenPair objectAtIndex: 1];
	float time;

	[[RADisplaySwapper sharedSwapper] swapScreen: screenA withScreen: screenB animationTime: 0.25f];
}

@end
