//
//  RAWindowAnimation.m
//  MonitorSwapper
//
//  Created by Quentin Carnicelli on Sat Mar 27 2004.
//  Copyright (c) 2004 Quentin D. Carnicelli. All rights reserved.
//

#import "RAWindowAnimation.h"

@implementation RAWindowAnimationAction

- (void)setProgress: (float)percent
{
}

- (void)complete
{
}

@end

#pragma mark -

@implementation RAWindowAnimationMoveAction

- (id)initWithWindow: (NSWindow*)window newOrigin: (NSPoint)origin
{
	self = [super init];
	
	if( self )
	{
		mWindow = [window retain];
		mStartOrigin = [mWindow frame].origin;
		mEndOrigin = origin;
	}
	
	return self;
}

- (void)dealloc
{
	[mWindow release];
	[super dealloc];
}

- (void)setProgress: (float)percent
{
	NSPoint curOrigin;
	
	curOrigin.x = (mEndOrigin.x - mStartOrigin.x) * percent + mStartOrigin.x;
	curOrigin.y = (mEndOrigin.y - mStartOrigin.y) * percent + mStartOrigin.y;
	
	[mWindow setFrameOrigin: curOrigin];
}

- (void)complete
{
	[mWindow setFrameOrigin: mEndOrigin];
}

@end

#pragma mark -

@implementation RAWindowAnimationList

- (id)init
{
	self = [super init];
	
	if( self )
	{
		mActions = [[NSMutableArray alloc] init];
	}
	
	return self;
}

- (void)dealloc
{
	[mActions release];
	[super dealloc];
}

- (void)addAction: (RAWindowAnimationAction*)action
{
	[mActions addObject: action];
}

- (void)animate: (NSTimeInterval)duration
{
	NSTimeInterval now;
	NSTimeInterval start;
	float percent;
	int i;
	
	if( ![mActions count] )
		return;
	
	if( duration > 0 )
	{
		start = now = [NSDate timeIntervalSinceReferenceDate];
	
		while( (now - start) < duration )
		{
			percent = MAX( 0, MIN( 1, (now-start)/duration ) );
			
			for( i = 0; i < [mActions count]; i++ )
				[[mActions objectAtIndex: i] setProgress: percent];
			
			now = [NSDate timeIntervalSinceReferenceDate];
		}
	}
	
	for( i = 0; i < [mActions count]; i++ )
		[[mActions objectAtIndex: i] complete];
}

@end